/// A cell is a single FORTH stack value, all arguments and return values given to OpenFirmware
/// must fit in a cell.
pub type Cell = i32;

/// # DoubleWord
/// a structure containing a low and high cell
/// # example
/// ```
/// use ofw::cell_conv::DoubleWord;
/// let value: u64 = 0xabcdef0123456789;
/// let dw: DoubleWord = value.into();
/// assert_eq!(dw.high as u32, 0xabcdef01);
/// assert_eq!(dw.low as u32, 0x23456789);
/// ```
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct DoubleWord {
    pub low: Cell,
    pub high: Cell,
}

impl From<i64> for DoubleWord {
    fn from(dw: i64) -> Self {
        Self {
            low: dw as Cell,
            high: (dw >> 32) as Cell,
        }
    }
}

impl From<u64> for DoubleWord {
    fn from(dw: u64) -> Self {
        Self {
            low: dw as Cell,
            high: (dw >> 32) as Cell,
        }
    }
}

impl From<DoubleWord> for i64 {
    fn from(dw: DoubleWord) -> Self {
        (dw.high as i64) << 32 | dw.low as i64
    }
}

impl From<DoubleWord> for u64 {
    fn from(dw: DoubleWord) -> Self {
        (dw.high as u64) << 32 | dw.low as u64
    }
}